function TurboModule(global, env, buffer) {
"use asm";
    //##################################
    //#            RUNTIME             #
    //##################################
    //part of asm module, we have global, foreign, buffer somewhere above this code
    
    var HEAP8 = new global.Int8Array(buffer);
    var HEAP16 = new global.Int16Array(buffer);
    var HEAP32 = new global.Int32Array(buffer);
    var HEAPU8 = new global.Uint8Array(buffer);
    var HEAPU16 = new global.Uint16Array(buffer);
    var HEAPU32 = new global.Uint32Array(buffer);
    var HEAPF32 = new global.Float32Array(buffer);
    var HEAPF64 = new global.Float64Array(buffer);
    
    var NULL = 0;
    var STACKTOP=env.STACKTOP|0;
    var STACK_MAX=env.STACK_MAX|0;
    var fround = global.Math.fround;
    
    //##################################
    //#            IMPORTS             #
    //##################################
    var Math_abs = global.Math.abs;
    var Math_acos = global.Math.acos;
    var Math_asin = global.Math.asin;
    var Math_atan = global.Math.atan;
    var Math_atan2 = global.Math.atan2;
    var Math_ceil = global.Math.ceil;
    var Math_cos = global.Math.cos;
    var Math_exp = global.Math.exp;
    var Math_floor = global.Math.floor;
    var Math_log = global.Math.log;
    var Math_max = global.Math.max;
    var Math_min = global.Math.min;
    var Math_pow = global.Math.pow;
    var Math_sin = global.Math.sin;
    var Math_sqrt = global.Math.sqrt;
    var Math_tan = global.Math.tan;
    var Math_imul = global.Math.imul;
    
    //##################################
    //#             CODE               #
    //##################################
    function testI32(a, b, c, d) {
        a = a|0;
        b = b|0;
        c = c|0;
        d = d|0;
        return (Math_imul((a + b)|0, (c)|0) + d|0);
    }
    
    
    function testF32(a, b, c, d) {
        a = fround(a);
        b = fround(b);
        c = fround(c);
        d = fround(d);
        return fround((a + b) * (c + d));
    }
    
    
    function testF64(a, b, c, d) {
        a = +a;
        b = +b;
        c = +c;
        d = +d;
        return +((a + b) * (c + d));
    }
    
    
    return {
       testI32:testI32,
       testF32:testF32,
       testF64:testF64
    }
}
function TurboWrapper(exports, buffer) {

    exports.init_malloc();

    return {
        exports: exports,
        RAW_MEMORY: buffer,

        getMemoryUsage: function () {
            const top = Atomics.load(HEAP32, 2);
            // top -= freeMemory;
            return Math.fround(top / (1024 * 1024));
        }/*,
        HEAP8: new Int8Array(buffer),
        HEAP16: new Int16Array(buffer),
        HEAP32: new Int32Array(buffer),
        HEAPU8: new Uint8Array(buffer),
        HEAPU16: new Uint16Array(buffer),
        HEAPU32: new Uint32Array(buffer),
        HEAPF32: new Float32Array(buffer),
        HEAPF64: new Float64Array(buffer)*/

    }
}
function initTurbo(bytes) {
    var buffer = new ArrayBuffer(bytes);

    if (buffer.byteLength < 16) {
        throw new Error("The memory is too small even for metadata");
    }

    return TurboWrapper(TurboModule(
        typeof global !== 'undefined' ? global : window,
        typeof env !== 'undefined' ? env : {
            STACKTOP: 8,
            STACK_MAX: 8
        },
        buffer
    ), buffer);
}
